@@ -134,6 +134,74 @@ function Debugger(sel) {
 
         });
 
+        // Keybindings
+        // Seems that divs can't receive key events, so we can't bind the
+        // handler to this.elt :(
+        var lastControlKeyElement = undefined;
+        $(document).keypress(function(e) {
+            //if (e.currentTarget !== document) {
+            if (! $(e.target).is('body')) {
+                // Don't handle events for elements like text inputs
+                return;
+            }
+            switch (e.keyCode) {
+                case 115: // s
+                    //lastControlKeyElement = $('[data-action="stepin"]');
+                    lastControlKeyElement = dbg.controlButtons.filter('[data-action="stepin"]')
+                            .click();
+                    e.stopPropagation();
+                    break;
+                case 110: // n
+                    lastControlKeyElement = dbg.controlButtons.filter('[data-action="stepover"]')
+                        .click();
+                    e.stopPropagation();
+                    break;
+                case 114: // r
+                     dbg.controlButtons.filter('[data-action="stepout"]')
+                                        .click();
+                    e.stopPropagation();
+                    break;
+                case 99: // c
+                    dbg.controlButtons.filter('[data-action="continue"]')
+                                        .click();
+                    e.stopPropagation();
+                    break;
+                case 113: // q
+                    dbg.controlButtons.filter('[data-action="exit"]')
+                                        .click();
+                    e.stopPropagation();
+                    break;
+                case 120: // x
+                    $('#add-watch-expr').click();
+                    e.stopPropagation();
+                    e.preventDefault();
+                    break;
+                case 13:  // cr
+                    if (lastControlKeyElement !== undefined) {
+                        lastControlKeyElement.click();
+                    }
+                    e.stopPropagation();
+                    break;
+                case 46: // .
+                    dbg.stackDiv.find('.tab-pane.active .current-sub-and-args').click();
+                    e.stopPropagation();
+                    break;
+                case 102: // f
+                    $('#add-file').click();
+                    e.stopPropagation();
+                    break;
+                case 76: // L
+                    $('#breakpoint-container-handle').click();
+                    e.stopPropagation();
+                    break;
+                case 98: //b
+                    dbg.stackDiv.find('.tab-pane.active .code-line.active span.lineno').click();
+                    e.stopPropagation();
+                    break;
+            }
+        });
+
+
         this.watchDiv.on('click', '.remove-watched-expression',
                     this.removeWatchExpression.bind(this));
         this.watchDiv.on('click', '.expr-collapse-button',
@@ -678,9 +746,9 @@ function Debugger(sel) {
         var isVisible = ((elemBottom >= containerTop) && (elemTop <= containerBottom)
                         && (elemBottom <= containerBottom) &&  (elemTop >= containerTop) );
         if (!isVisible) {
-            // If the target element is off the top, leave 2 lines at the top,
-            // if it's off the bottom, leave 2 lines at the bottom
-            var overShoot = (elemBottom < containerTop) ? 2 : -2;
+            //var overShoot = (elemBottom < containerTop) ? -4 : 4;
+            // Make it the 4th line from the top
+            var overShoot = -4;
             container.scrollTo(activeLine, { over: overShoot });
         }
         return activeLine;
